#include <stdlib.h>
#include <allegro.h>
#include "052ct.h"

/* Ok, take a look at the triangle_init function. Each triangle has 3 vertices
 * but check that we only allow the user to specify one coordinate. This is
 * because I wanted to make this function decide the shape of the triangle
 * at random.  It stores the relative positions of the vertices around the
 * given point.
 * The speed is also decided by the function.
 */

TRIANGLE_T *tri_init(int new_x, int new_y, int new_col)
{
    TRIANGLE_T *temp;

    // Allocate memory for the circle. If unsuccessful, return NULL.
    temp = malloc(sizeof(TRIANGLE_T));

    if (!temp)
	return NULL;

    /* This is the reference position */
    temp->x = new_x;
    temp->y = new_y;

    temp->col = new_col;
    temp->dx = temp->dy = temp->xs = temp->ys = 0;

    // We decide the speed.Remember we want triangles to move only in one axis
    // First decide axis, if 0, horizontal, else vertical
    if (rand() % 2 == 0) {
	temp->ys = 0;
	while (temp->xs == 0)
	    temp->xs = (rand() % 7) - 3;
    } else {
	temp->xs = 0;
	while (temp->ys == 0)
	    temp->ys = (rand() % 7) - 3;
    }

    // And now we decide the relative vertex locations
    temp->x1 = -40 + rand() % 80;
    temp->x2 = -40 + rand() % 80;
    temp->x3 = -40 + rand() % 80;
    temp->y1 = -40 + rand() % 80;
    temp->y2 = -40 + rand() % 80;
    temp->y3 = -40 + rand() % 80;

    return temp;
}

void tri_destroy(TRIANGLE_T * tri)
{
    if (tri)
	free(tri);
}

void tri_draw(TRIANGLE_T * tri, BITMAP * bmp)
{
    triangle(bmp, tri->x + tri->x1, tri->y + tri->y1,
	     tri->x + tri->x2, tri->y + tri->y2,
	     tri->x + tri->x3, tri->y + tri->y3, tri->col);

    tri->dx = tri->x;
    tri->dy = tri->y;
}

void tri_update(TRIANGLE_T * tri)
{
    // First we update the position of the triangle...
    tri->x += tri->xs;
    tri->y += tri->ys;

    /* To make the triangles go off one side and come on the other, we
     * let them go right off the screen and then make them jump just off
     * the opposite side. */

    if (tri->x > SCREEN_W + 40)
	tri->x -= SCREEN_W + 80;

    if (tri->y > SCREEN_H + 40)
	tri->y -= SCREEN_H + 80;

    if (tri->x < -40)
	tri->x += SCREEN_W + 80;

    if (tri->y < -40)
	tri->y += SCREEN_H + 80;
}

void tri_erase(TRIANGLE_T * tri, BITMAP * bmp)
{
    triangle(bmp, tri->dx + tri->x1, tri->dy + tri->y1,
	     tri->dx + tri->x2, tri->dy + tri->y2,
	     tri->dx + tri->x3, tri->dy + tri->y3, 0);
}
