#include <stdio.h>
#include <allegro.h>
#include "0623.h"
#include "0623c.h"

// Number of max pixels per frame of circle movement
#define  JOYSPEED       5

int end_game;			/* flag we'll set to end the game */
int pressed_fire_button;	// This wii help with the color feature

BITMAP *dblbuffer;

CIRCLE_T *moving_circle;

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown();			/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    int i;

    install_keyboard();

    // First call to init_joystick checks to see it there's any at all...
    if (install_joystick(JOY_TYPE_AUTODETECT) != 0) {
	printf("Sorry, didn't detect any joystick\n");
	exit(1);
    }
    printf("Center stick 1 and press a button\n");
    readkey();

    install_joystick(JOY_TYPE_AUTODETECT);

    for (i = 0; i < 1; i++) {
	while (joy[i].flags & JOYFLAG_CALIBRATE) {
	    char *msg = calibrate_joystick_name(i);
	    printf("%s, and press a key\n", msg);
	    readkey();
	    if (calibrate_joystick(i) != 0) {
		printf("oops!\n");
		exit(2);
	    }
	}

	printf("Now center the stick and press a key\n");
	readkey();
    }

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	printf("%s\n", allegro_error);
	exit(3);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	printf("Sorry, not enough memory");
	exit(4);
    }
    clear(dblbuffer);

    moving_circle = circle_init(SCREEN_W / 2, SCREEN_H / 2, 50, 3, 0, 0);
    if (moving_circle == NULL) {
	allegro_exit();
	printf("Sorry, not enough memory");
	exit(5);
    }
    pressed_fire_button = 0;

    circle_draw(moving_circle, dblbuffer);
    show_double_buffer();
}

void input(void)
{
    if (key[KEY_ESC])
	end_game++;

    poll_joystick();

    /* Simple trick. Since we want to change the color only when the joystick
     * button state changes, we store in pressed_fire_button the actual
     * value of the button, and compare it in next loops. If it's different,
     * then we change color and update our variable. We also make sure that
     * the new color will be different from the old one using the global
     * variable as a temporary holder of the color and comparing it with the
     * new rand value.
     */

    if (joy_b1 != pressed_fire_button) {
	pressed_fire_button = moving_circle->col;
	while (pressed_fire_button == moving_circle->col)
	    moving_circle->col = rand() % 14;

	pressed_fire_button = joy_b1;
    }
    moving_circle->xs += joy_x / (128 / JOYSPEED);
    moving_circle->ys += joy_y / (128 / JOYSPEED);
/*
   if (joy_left) moving_circle->xs--;  // if the user is pressing left
   if (joy_right) moving_circle->xs++; // arrow, go left same for right
   if (joy_up) moving_circle->ys--;    // same for up; note that y
   if (joy_down) moving_circle->ys++;  // _decreases_ to go up and for
   // down y _increases_
 */
}

void process(void)
{
    circle_update(moving_circle);
}

void output(void)
{
    circle_erase(moving_circle, dblbuffer);
    circle_draw(moving_circle, dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    circle_destroy(moving_circle);
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
