// Main input routines. This is the file you should look at carefully

#include <stdio.h>
#include <allegro.h>
#include "circle.h"
#include "main.h"
#include "input.h"
#include "gui.h"

// These defines help knowing what type of input we are using...

#define KEYBOARD  1
#define MOUSE     2
#define JOYSTICK  3

//Here come internal structures...

struct key_settings_t
{
  int left_key, right_key, up_key, down_key,fire1_key, fire2_key;
};

struct joy_settings_t
{
  int empty;  /* You could place here the functions of the joy's buttons,
               * for example. Say, you let the user configure them, so that
               * button 1 is not fire, but special attack, etc. Or
               * You could specify here the active stick, since there could
               * be more than one joysticks plugged in.
               */
};

struct mos_settings_t
{
  int speed;   // Mouse speed.
};

typedef struct internal_input_t INTERNAL_INPUT_T;

struct internal_input_t
{
  int type;       // Keyboard, joystick, mouse, etc...
  union {
    struct key_settings_t keyboard_settings;
    struct joy_settings_t joystick_settings;
    struct mos_settings_t mouse_settings;
  } info;                         // Info union (use memory to the max :-)
  struct input_t *public_struct;  // Pointer to general movement struct
};

// And this will be an internal input linked list.

typedef struct i_device_t I_DEVICE_T;

struct i_device_t
{
  INTERNAL_INPUT_T *i_input;
  I_DEVICE_T *next;
};

I_DEVICE_T *ibase;

int is_mouse,is_joy;  // Are there any mouse or joysticks installed?

INPUT_T *input_create_mouse (int max_mousespeed)
{
  I_DEVICE_T *i;
  INPUT_T *temp;

  temp = malloc ( sizeof (struct input_t) );
  if (!temp)
    return NULL;

  temp->dx = temp->dy = temp->fire1 = temp-> fire2 = 0;

  // Now we add one internal struct holding the key values.
  i = ibase;
  while (i->next != NULL) i=i->next;
  i->next = malloc ( sizeof (struct i_device_t) );
  if (!i->next) {
    free(temp);
    return NULL;
  }
  i = i->next;
  i->next = NULL;

  i->i_input = malloc (sizeof (struct internal_input_t) );
  if (!i->i_input) {
    free (i);
    free (temp);
    return NULL;
  }

  i->i_input->type = MOUSE;
  i->i_input->public_struct = temp;
  i->i_input->info.mouse_settings.speed = max_mousespeed;


  return temp;
}

INPUT_T *input_create_joystick (void)
{
  I_DEVICE_T *i;
  INPUT_T *temp;

  temp = malloc ( sizeof (struct input_t) );
  if (!temp)
    return NULL;

  temp->dx = temp->dy = temp->fire1 = temp-> fire2 = 0;

  // Now we add one internal struct
  i = ibase;
  while (i->next != NULL) i=i->next;
  i->next = malloc ( sizeof (struct i_device_t) );
  if (!i->next) {
    free(temp);
    return NULL;
  }
  i = i->next;
  i->next = NULL;

  i->i_input = malloc (sizeof (struct internal_input_t) );
  if (!i->i_input) {
    free (i);
    free (temp);
    return NULL;
  }

  i->i_input->type = JOYSTICK;
  i->i_input->public_struct = temp;

  return temp;
}

INPUT_T *input_create_keybd (int left_k, int right_k,
                             int up_k, int down_k,
                             int fire1_k, int fire2_k)
{
  I_DEVICE_T *i;
  INPUT_T *temp;

  // First we set up the general movement struct.
  temp = malloc ( sizeof (struct input_t) );
  if (!temp)
    return NULL;

  temp->dx = temp->dy = temp->fire1 = temp-> fire2 = 0;

  // Now we add one internal struct holding the key values.
  i = ibase;
  while (i->next != NULL) i=i->next;
  i->next = malloc ( sizeof (struct i_device_t) );
  if (!i->next) {
    free(temp);
    return NULL;
  }
  i = i->next;
  i->next = NULL;

  i->i_input = malloc (sizeof (struct internal_input_t) );
  if (!i->i_input) {
    free (i);
    free (temp);
    return NULL;
  }

  i->i_input->type = KEYBOARD;
  i->i_input->public_struct = temp;
  i->i_input->info.keyboard_settings.left_key  = left_k;
  i->i_input->info.keyboard_settings.right_key = right_k;
  i->i_input->info.keyboard_settings.up_key    = up_k;
  i->i_input->info.keyboard_settings.down_key  = down_k;
  i->i_input->info.keyboard_settings.fire1_key = fire1_k;
  i->i_input->info.keyboard_settings.fire2_key = fire2_k;

  return temp;
}

void calibrate_our_joystick(void)
{
  int i;

  // If a joystick is detected, calibrate it...
  if (is_joy == 0 && num_joysticks >=1) {
    remove_joystick();
    clear(screen);
    textout_centre(screen,font,"Center stick 1 and press a button",
              SCREEN_W>>1,SCREEN_H>>1,15);
    readkey();

    install_joystick(JOY_TYPE_AUTODETECT);
   
    for (i=0; i<1; i++) {
      while (joy[i].flags & JOYFLAG_CALIBRATE) {
        char *msg = calibrate_joystick_name(i);
        clear(screen);
        textprintf_centre(screen,font,SCREEN_W>>1,SCREEN_H>>1,
                          15,"%s, and press a key", msg);
        readkey();
        if (calibrate_joystick(i) != 0) {
          clear(screen);
          textout_centre(screen,font,"oops",
                         SCREEN_W>>1,SCREEN_H>>1,15);
          return;
        }
      }
    
      clear(screen);
      textout_centre(screen,font,"Now center the stick and press a key",
                     SCREEN_W>>1,SCREEN_H>>1,15);
      readkey();
    }

  }
  else {
    clear_keybuf();
    clear(screen);
    textout_centre(screen,font,"No joystick detected",
                   SCREEN_W>>1,SCREEN_H>>1,15);
    readkey();
  }
}

void redefine_keyboard(void)
{
  int f,i, keys[6];
  char texts[6][10]={"left","right","up","down","fire 1","fire 2"};
  char temp_text[100];
  I_DEVICE_T *t;

  for (f = 0; f < 6; f++) {
    sprintf(temp_text,"Press a key for %s",texts[f]);

    clear(screen);
    textout_centre(screen,font,temp_text,SCREEN_W>>1,SCREEN_H>>1,15);
    clear_keybuf();

    // First wait for a keypress
    while(!keypressed());
    // Now read scan the keycodes
    for (i=1; i < 128; i++) {
      if (key[i])
        keys[f] = i;
    }
  }

  // Now search the device which is a keyboard in the internal list...
  t = ibase->next;

  while (t->i_input->type != KEYBOARD && t != NULL)
    t = t->next;

  t->i_input->info.keyboard_settings.left_key  = keys[0];
  t->i_input->info.keyboard_settings.right_key = keys[1];
  t->i_input->info.keyboard_settings.up_key    = keys[2];
  t->i_input->info.keyboard_settings.down_key  = keys[3];
  t->i_input->info.keyboard_settings.fire1_key = keys[4];
  t->i_input->info.keyboard_settings.fire2_key = keys[5];

  clear_keybuf();
}

int get_mouse_speed(void)
{
  I_DEVICE_T *t;

  t = ibase->next;

  while (t->i_input->type != MOUSE) {
    if (t==NULL)
      return 1; // Return a simple value if there's no mouse at all.
    t = t->next;
  }

  return t->i_input->info.mouse_settings.speed;
}

void change_mouse_speed(int speed)
{
  I_DEVICE_T *t;

  t = ibase->next;

  while (t->i_input->type != MOUSE) {
    if (t==NULL)
      return; // Return. There's no mouse.
    t = t->next;
  }

  t->i_input->info.mouse_settings.speed = speed;
}

/* This function not only destroys the public structure, but it has to
 * search through all the internal structures, find the correct one, and
 * delete it too.
 */
void input_destroy (INPUT_T *what)
{
  I_DEVICE_T *t,*p;

  if (!what) return;  // There's nothing to remove!!

  // Find the internal structure with the same public pointer...
  t = ibase->next;
  p = ibase;
  while (t->i_input->public_struct != what && t!=NULL) {
    t = t->next;  // Advance each pointer one step...
    p = p->next;
  }

  if (t==NULL)
    exit(99); // Error, you should never reach here.

  // Found item. First remove public structure...
  free (what);
  // Now remove the internal input structure
  free (t->i_input);
  // Free this item in the linked list
  p->next = t->next;
  free (t);
}


void input_shutdown (void)
{
  input_destroy(all_input[0]);
  input_destroy(all_input[1]);
  input_destroy(all_input[2]);
  remove_keyboard();
  remove_joystick();
  remove_mouse();
  // Free internal dummy node
  free(ibase);
}

int input_init (void)
{
// int i;

  all_input[0] = all_input[1] = all_input[2] = NULL;

  // Create a dummy node for the internal input list.
  ibase = malloc (sizeof (I_DEVICE_T) );
  if (!ibase)
    return 3;
  // Clean it.
  ibase->i_input = NULL;
  ibase->next = NULL;

  install_keyboard();
  is_mouse = install_mouse();
  is_joy = install_joystick(JOY_TYPE_AUTODETECT);

  all_input[0] = input_create_keybd (KEY_LEFT, KEY_RIGHT, KEY_UP,
                                     KEY_DOWN, KEY_ENTER, KEY_SPACE);
  if (is_joy == 0)
    all_input[1] = input_create_joystick();

  if (is_mouse >= 1)
    all_input[2] = input_create_mouse(3);

  return 0;
}

void input(void)
{
  INTERNAL_INPUT_T *ii;
  I_DEVICE_T *dev;
  int dx,dy;

  if(key[KEY_ESC])
    end_game += menu();

  dev = ibase->next;

  while (dev != NULL) {

    ii = dev->i_input;

    switch (ii->type) {
      case KEYBOARD:
        ii->public_struct->dx = (key[ii->info.keyboard_settings.left_key] ? -128 : 0)
                              + (key[ii->info.keyboard_settings.right_key] ? 128 : 0);
        ii->public_struct->dy = (key[ii->info.keyboard_settings.up_key] ? -128 : 0)
                              + (key[ii->info.keyboard_settings.down_key] ? 128 : 0);
        ii->public_struct->fire1 = key[ii->info.keyboard_settings.fire1_key];
        ii->public_struct->fire2 = key[ii->info.keyboard_settings.fire2_key];
        break;
      case JOYSTICK:
        poll_joystick();
        ii->public_struct->dx = joy_x;
        ii->public_struct->dy = joy_y;
        ii->public_struct->fire1 = joy_b1;
        ii->public_struct->fire2 = joy_b2;
        break;
      case MOUSE:
        get_mouse_mickeys(&dx,&dy);
        ii->public_struct->dx = dx*128/ii->info.mouse_settings.speed;
        ii->public_struct->dy = dy*128/ii->info.mouse_settings.speed;
        ii->public_struct->fire1 = mouse_b & 1;
        ii->public_struct->fire2 = mouse_b & 2;
        break;
      default:
        break;
    }

    dev = dev->next;
  }
}
